
/* components defined in this file */
const CLINE_SERVICE_CTRID =
    "@mozilla.org/commandlinehandler/general-startup;1?type=jslib";
const CLINE_SERVICE_CID =
    Components.ID("{b9d7bca5-ddc9-4151-b1cd-f33347addf9b}");

const JSLIBCNT_HANDLER_CONTRACTID =
    "@mozilla.org/uriloader/content-handler;1?type=x-application-jslib";
const JSLIBCNT_HANDLER_CID =
    Components.ID("{764a1404-3ac6-4d66-bf7e-a1c5fe1dccd0}");

const JSLIBPROT_HANDLER_CONTRACTID =
    "@mozilla.org/network/protocol;1?name=jslib";
const JSLIBPROT_HANDLER_CID =
    Components.ID("{119f6754-ba12-4aed-b3ee-d409895e77ba}");

/* components used by this file */
const ASS_CONTRACTID =
    "@mozilla.org/appshell/appShellService;1";

const MEDIATOR_CONTRACTID =
    "@mozilla.org/appshell/window-mediator;1";

const CATMAN_CTRID = "@mozilla.org/categorymanager;1";

const STANDARDURL_CONTRACTID =
    "@mozilla.org/network/standard-url;1";

const IOSERVICE_CONTRACTID = 
    "@mozilla.org/network/io-service;1";


/* interafces used in this file */
const nsIWindowMediator  = Components.interfaces.nsIWindowMediator;
const nsICmdLineHandler  = Components.interfaces.nsICmdLineHandler;
const nsICategoryManager = Components.interfaces.nsICategoryManager;
const nsIContentHandler  = Components.interfaces.nsIContentHandler;
const nsISupports        = Components.interfaces.nsISupports;
const nsIURI             = Components.interfaces.nsIURI;
const nsIStandardURL     = Components.interfaces.nsIStandardURL;
const nsIChannel         = Components.interfaces.nsIChannel;
const nsIProtocolHandler = Components.interfaces.nsIProtocolHandler;
const nsIRequest         = Components.interfaces.nsIRequest;
const nsIIOService       = Components.interfaces.nsIIOService;
const nsIAppShellService = Components.interfaces.nsIAppShellService;


/* Command Line handler service */
function CLineService() {}

CLineService.prototype.commandLineArgument = "-jslib";
CLineService.prototype.prefNameForStartup = "general.startup.jslib";
CLineService.prototype.chromeUrlForTask = "chrome://jslib/content/";
CLineService.prototype.helpText = "Start jsLib";
CLineService.prototype.handlesArgs = true;
CLineService.prototype.defaultArgs = "";
CLineService.prototype.openWindowWithArgs = true;

/* factory for command line handler service (CLineService) */
var CLineFactory = new Object();

CLineFactory.createInstance =
function clf_create (outer, iid) {
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    if (!iid.equals(nsICmdLineHandler) && !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_INVALID_ARG;

    return new CLineService();
}

/* x-application-jslib content handler */
function jslibContentHandler ()
{}

jslibContentHandler.prototype.QueryInterface =
function (iid)
{
    if (!iid.equals(nsIContentHandler))
        throw Components.results.NS_ERROR_NO_INTERFACE;

    return this;
}

jslibContentHandler.prototype.handleContent =
function (contentType, command, windowTarget, request)
{
    var e;
    var channel = request.QueryInterface(nsIChannel);
    
    var windowManager =
        Components.classes[MEDIATOR_CONTRACTID].getService(nsIWindowMediator);

    var w = windowManager.getMostRecentWindow("navigator:browser");

    if (w) {
        w.focus();
        w.loadURI("http://jslib.mozdev.org/");
    } else {
        var ass =
            Components.classes[ASS_CONTRACTID].getService(nsIAppShellService);
        w = ass.hiddenDOMWindow;

        var args = new Object ();
        args.url = "http://jslib.mozdev.org/";

        w.openDialog("chrome://navigator/content/", "_blank",
                     "chrome,menubar,toolbar,status,resizable,dialog=no",
                     args);
    }
}

/* content handler factory object (jslibContentHandler) */
var jslibContentHandlerFactory = new Object();

jslibContentHandlerFactory.createInstance =
function (outer, iid)
{
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    if (!iid.equals(nsIContentHandler) && !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_INVALID_ARG;

    return new jslibContentHandler();
}

/* jslib protocol handler component */
function jslibProtocolHandler()
{
}

jslibProtocolHandler.prototype.scheme = "jslib";
jslibProtocolHandler.prototype.defaultPort = 4100;
jslibProtocolHandler.prototype.protocolFlags = 
                   nsIProtocolHandler.URI_NORELATIVE |
                   nsIProtocolHandler.ALLOWS_PROXY;

jslibProtocolHandler.prototype.allowPort =
function (port, scheme)
{
    return false;
}

jslibProtocolHandler.prototype.newURI =
function (spec, charset, baseURI)
{
    var cls = Components.classes[STANDARDURL_CONTRACTID];
    var url = cls.createInstance(nsIStandardURL);
    url.init(nsIStandardURL.URLTYPE_STANDARD, 4100, spec, charset, baseURI);

    return url.QueryInterface(nsIURI);
}

jslibProtocolHandler.prototype.newChannel =
function (URI)
{
    ios = Components.classes[IOSERVICE_CONTRACTID].getService(nsIIOService);
    if (!ios.allowPort(URI.port, URI.scheme))
        throw Components.results.NS_ERROR_FAILURE;

    return new BogusChannel (URI);
}

/* protocol handler factory object (jslibProtocolHandler) */
var jslibProtocolHandlerFactory = new Object();

jslibProtocolHandlerFactory.createInstance =
function (outer, iid)
{
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    if (!iid.equals(nsIProtocolHandler) && !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_INVALID_ARG;

    return new jslibProtocolHandler();
}

/* bogus channel used by the JSLIBProtocolHandler */
function BogusChannel (URI)
{
    this.URI = URI;
    this.originalURI = URI;
}

BogusChannel.prototype.QueryInterface =
function (iid)
{
    if (!iid.equals(nsIChannel) && !iid.equals(nsIRequest) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;

    return this;
}

/* nsIChannel */
BogusChannel.prototype.loadAttributes = null;
BogusChannel.prototype.contentType = "x-application-jslib";
BogusChannel.prototype.contentLength = 0;
BogusChannel.prototype.owner = null;
BogusChannel.prototype.loadGroup = null;
BogusChannel.prototype.notificationCallbacks = null;
BogusChannel.prototype.securityInfo = null;

BogusChannel.prototype.open =
BogusChannel.prototype.asyncOpen =
function ()
{
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
}

BogusChannel.prototype.asyncOpen =
function (observer, ctxt)
{
    observer.onStartRequest (this, ctxt);
}

BogusChannel.prototype.asyncRead =
function (listener, ctxt)
{
    return listener.onStartRequest (this, ctxt);
}

/* nsIRequest */
BogusChannel.prototype.isPending =
function ()
{
    return true;
}

BogusChannel.prototype.status = Components.results.NS_OK;

BogusChannel.prototype.cancel =
function (status)
{
    this.status = status;
}

BogusChannel.prototype.suspend =
BogusChannel.prototype.resume =
function ()
{
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
}

/*****************************************************************************/

var Module = new Object();

Module.registerSelf =
function (cm, fileSpec, location, type)
{
    debug("*** Registering -jslib handler.\n");
    
    cm = cm.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    cm.registerFactoryLocation(CLINE_SERVICE_CID,
                                    "jsLib CommandLine Service",
                                    CLINE_SERVICE_CTRID, 
                                    fileSpec,
                                    location, 
                                    type);

	  catman = Components.classes[CATMAN_CTRID].getService(nsICategoryManager);
	  catman.addCategoryEntry("command-line-argument-handlers",
                            "jslib command line handler",
                            CLINE_SERVICE_CTRID, true, true);

    debug("*** Registering x-application-jslib handler.\n");
    cm.registerFactoryLocation(JSLIBCNT_HANDLER_CID,
                               "jsLib Content Handler",
                               JSLIBCNT_HANDLER_CONTRACTID, 
                               fileSpec,
                               location, 
                               type);

    debug("*** Registering jslib protocol handler.\n");
    cm.registerFactoryLocation(JSLIBPROT_HANDLER_CID,
                               "jsLib protocol handler",
                               JSLIBPROT_HANDLER_CONTRACTID, 
                               fileSpec, 
                               location,
                               type);

}

Module.unregisterSelf =
function(cm, fileSpec, location)
{
    cm = cm.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    cm.unregisterFactoryLocation(CLINE_SERVICE_CID, fileSpec);
	  catman = Components.classes[CATMAN_CTRID].getService(nsICategoryManager);
	  catman.deleteCategoryEntry("command-line-argument-handlers",
                                CLINE_SERVICE_CTRID, true);
}

Module.getClassObject =
function (cm, cid, iid) 
{
    if (cid.equals(CLINE_SERVICE_CID))
        return CLineFactory;

    if (cid.equals(JSLIBCNT_HANDLER_CID))
        return jslibContentHandlerFactory;

    if (cid.equals(JSLIBPROT_HANDLER_CID))
        return jslibProtocolHandlerFactory;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    throw Components.results.NS_ERROR_NO_INTERFACE;
    
}

Module.canUnload =
function(cm) { return true; }

/* entrypoint */
function NSGetModule(cm, fileSpec) { return Module; }
