/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is JSIRC Library
 *
 * The Initial Developer of the Original Code is New Dimensions Consulting,
 * Inc. Portions created by New Dimensions Consulting, Inc. are
 * Copyright (C) 1999 New Dimenstions Consulting, Inc. All
 * Rights Reserved.
 *
 * Contributor(s):
 *  Robert Ginda, rginda@ndcico.com, original author
 *  Peter Van der Beken, peter.vanderbeken@pandora.be, necko-only version
 *  Jeremie Miller, jer@jeremie.com, generalized into js socket lib (for Jabber)
 *

  Modified:
    Sep-2003  sdk updated to working (more) state.
          using: chatzilla/connection-xpcom.js

  Attributes:
    nsISocketTransport.openInputStream arguments:
    socket.openInputFlags = 0;          open flags      
    socket.openInputSegmentSize = 0;      segmentSize
    socket.openInputSegmentCount = 0;     segmentCount
  
    nsISocketTransport.openOutputStream arguments:
    socket.openOutputFlags = 0;         open flags
    socket.openOutputSegmentSize = 0;     segmentSize
    socket.openOutputSegmentCount = 0;      segmentCount
 */

if (typeof(JS_LIB_LOADED)=='boolean') {

var jsenv = new Object();
jsenv.HAS_STREAM_PROVIDER = ('nsIStreamProvider' in C.interfaces);

/*
  function:  Socket
  purpose:  Constructor for an instance of Socket
  interface:  
    socket( [ classID ] )

  arguments:
    classID : classesByID class-identifier

  returns:
    not applicable

  Note:  by default, Socket uses the local socket-transport-service, however
  callers looking for a specific implementation can pass in the class
  identifier as an alternative.
*/
function Socket ()
{
  this.openInputFlags = 0;
  this.openInputSegmentSize = 0;
  this.openInputSegmentCount = 0;

  this.openOutputFlags = 0;
  this.openOutputSegmentSize = 0;
  this.openOutputSegmentCount = 0;

  var defaultContractID = "@mozilla.org/network/socket-transport-service;1";
//  var defaultClassID = "{c07e81e0-ef12-11d2-92b6-00105a1b0d64}"

    var socketServiceClass;
  switch( arguments.length )
  {
    case 0:
          socketServiceClass = C.classes[ defaultContractID ];
      break;

    case 1:
      socketServiceClass =
        C.classesByID[ arguments[ 1 ] ];
      break;

    default:
      throw( "Socket: unexpected arguments for constructor" );
      break;
  }
  if ( arguments.length == 1 )

    
    if (!socketServiceClass)
        throw ("Socket constructor: Couldn't get socket service class.");
    
    var socketService = socketServiceClass.getService();
    if (!socketService)
        throw ("Socket constructor: Couldn't get socket service.");

    this._socketService = jslibQI(socketService, "nsISocketTransportService");

    // to preserve ourselves within necko/async
    this.wrappedJSObject = this;
}

/*
  function: open
  purpose:  opens the socket
  interface:  
    open( host, port [ binary ] )

  Arguments:
    host : String, host to connect to
    port : int, port number to use
    binary : optional, use binary input support, defaults false
    
  returns:
    true.
*/
Socket.prototype.open = function (host, port)
{
    this.host = host.toLowerCase();
    this.port = port;
  this.binary = ( arguments.length > 2 ) ? arguments[ 2 ] : false;

  // in theory, we'd look up proxy information here.  but we're being
  // a bare socket so.... 
  
  // create the transport:
  // socketTypes = null
  // typeCount = 0
  // host
  // port
  // proxy-info = null
    this._transport = this._socketService.createTransport ( null, 0, host, port, null );
    if (!this._transport)
        throw ("Socket.open: Error opening transport.");

  var openFlags = ( this.blocking ) && ( typeof document == "object" )
        ? 0
        : C.interfaces.nsITransport.OPEN_BLOCKING;

  this._inputStream = this._transport.openInputStream( 
            this.openInputFlags, 
            this.openInputSegmentSize, 
            this.openInputSegmentCount );
  if (!this._inputStream)
    throw ("Socket.open: Error getting input stream.");

  if ( this.binary )
    this._inputInterface = this.toBinaryInputStream( this._inputStream );
  else
    this._inputInterface = this.toScriptableInputStream( this._inputStream );

    this._outputStream = this._transport.openOutputStream( 
            this.openOutputFlags, 
            this.openOutputSegmentSize, 
            this.openOutputSegmentCount );
    if (!this._outputStream)
        throw ("Socket.open: Error getting output stream.");

  // we really should call _transport.isAlive (?) but that always
  // returns false.  attempts to use "available()" don't fail (but will
  // later if a connection isn't open).
    this.isConnected = true;

    return this.isConnected;
}

/*
  function:  close
  purpose:  closes the socket connection
  interface:
    close()

  returns:
    nothing.
*/
Socket.prototype.close = function ()
{
  this.isConnected = false;

  // calls to _inputStream.close() and _outputStream.close() didn't 
  // function.
  this._transport.close( 0 );
}

/*
  function: write
  purpose:
    writes the given string to the socket.

  interface:
    write( str )

  arguments:
    str : string to be written.

  returns:
    true if no exception is thrown.
*/
Socket.prototype.write = function (str)
{
    if (!this.isConnected)
        throw "Socket.write: Not Connected.";

    var rv = false;

    try
    {
        this._outputStream.write(str, str.length);
        rv = true;
    }
    catch (ex)
    {
        if (typeof ex != "undefined")
        {
            this.isConnected = false;
            throw (ex);
        }
        else
            rv = false;
    }
    
    return rv;
}

/*
    function: read
    purpose:
    reads data from a socket.

    interface:
    read( bytes )

  arguments:
    bytes : integer, number of bytes to read in

    returns:
    string

  Note:
    will only return the smaller of specified vs available bytes.

*/
Socket.prototype.read = function ( bytes )
{
    if (!this.isConnected)
        throw "Socket.read: Not Connected.";
    
    var returnValue = new String;

    try
    {
    // attempts to read more bytes than available will throw
    // an exception.
    bytes = Math.min( this._inputInterface.available(), bytes );
    if ( bytes )
      if ( this.binary )
        // despite the documentation, this call works
        returnValue = this._inputInterface.readBytes( bytes );
      else
        returnValue = this._inputInterface.read( bytes );
        else
            returnValue = "";
    }
    catch (ex)
    {
    // the assumption is that we lost the connection...
        if (typeof ex != "undefined") {
            this.isConnected = false;
            throw (ex);
        } else {
            returnValue = "";
        }
    }
    
    return returnValue;
}

/*
    function: available
    purpose:
    number of bytes waiting to be read

    interface:
    available()

    returns:
    integer
*/
Socket.prototype.available = function ()
{
  if ( !this.isConnected )
    throw "Socket.available:  Not Connected.";

  return this._inputInterface.available();
}

/*
  function:  toBinaryInputStream
  purpose: creates an nsIBinaryInputStream wrapper around the given
  inputStream.

  interface:
      toBinaryInputStream( inputStream )

  arguments:
    inputStream : result of openInputStream

  returns:
    nsIBinaryInputStream
*/
Socket.prototype.toBinaryInputStream = function( inputStream )
{
    var si = jslibCreateInstance("@mozilla.org/binaryinputstream;1", "nsIBinaryInputStream");
    si.setInputStream( inputStream );

    return si;
}

/*
    function: toScriptableInputStream
    purpose:
    creates an nsIScriptableInputStream wrapper around the given
    inputStream.

    interface:
    toScriptableInputStream( inputStream )

  arguments:
    inputStream : result of openInputStream

    returns:
    nsIScriptableInputStream
*/
Socket.prototype.toScriptableInputStream = function( inputStream )
{
    var si = jslibCreateInstance("@mozilla.org/scriptableinputstream;1", 
                                 "nsIScriptableInputStream");
    si.init( inputStream );

    return si;
}

// sdk/16-Sep-2003:  async is non-functional
/*
    function: async
    purpose:
    interface:

    returns:
    usage:

     var aSocket = new Socket();
     var observer = {
        streamStarted: function(socketContext){ },           //onstart action
        streamStopped: function(socketContext, status){ },   //onstop action
        receiveData:   function(data){alert(data)}
      }
     aSocket.open("ftp.mozilla.org", 21);
     aSocket.async(observer );

*/
Socket.prototype.async = function (observer)
{
    this._pump = jslibCreateInstance("@mozilla.org/network/input-stream-pump;1",
                                     "nsIInputStreamPump");

    this._pump.init(this._inputStream, -1, -1, 0, 0, false);
    this._pump.asyncRead(new SocketListener(observer), this);
}


// async callbacks
function SocketListener( observer )
{
  this._observer = observer;
}

SocketListener.prototype.onStartRequest = 
function (channel, socketContext)
{
    socketContext = socketContext.wrappedJSObject;
  this._observer.streamStarted( socketContext );
}

SocketListener.prototype.onStopRequest = 
function (channel, socketContext, status, errorMsg)
{
    socketContext = socketContext.wrappedJSObject;
  this._observer.streamStopped( socketContext, status );
}

SocketListener.prototype.onDataAvailable = 
function (channel, socketContext, inStr, sourceOffset, count)
{
    socketContext = socketContext.wrappedJSObject;
  this._observer.receiveData( socketContext.read( count ) );
}

};  // END socket.js
